.mode-toggle {
  display: flex;
  gap: 16px;
  padding: 4px 16px 0;
  background: var(--bg-raised);
  border-bottom: 1px solid var(--border);
}

.mode-toggle-logo {
  font-weight: 700;
  font-size: 18px;
  font-family: var(--font-mono);
  color: var(--accent-blue);
  margin-right: 8px;
  padding: 10px 0;
}

.mode-toggle-btn {
  padding: 10px 0 6px;
  border: none;
  border-radius: 0;
  background: transparent;
  color: var(--text-secondary);
  font-size: 14px;
  font-weight: 600;
  cursor: pointer;
  border-bottom: 2px solid transparent;
  transition:
    color 0.15s,
    border-color 0.15s;
}

.mode-toggle-btn:hover {
  color: var(--text-primary);
}

.mode-toggle-btn:focus {
  outline: none;
  border-bottom-color: rgba(140, 100, 200, 0.6);
}

.mode-toggle-btn:focus:not(:focus-visible) {
  border-bottom-color: transparent;
}

.mode-toggle-btn:focus-visible {
  outline: none;
  border-bottom-color: rgba(140, 100, 200, 0.6);
}

.mode-toggle-btn.active,
.mode-toggle-btn.active:focus,
.mode-toggle-btn.active:focus-visible {
  color: var(--text-primary);
  border-bottom-color: var(--accent-blue);
}
