.state-filter {
  display: flex;
  flex-direction: column;
  height: 100%;
  overflow: hidden;
  background: var(--bg-deepest);
}

.state-filter-header {
  display: flex;
  align-items: center;
  padding: 8px 20px;
  border-bottom: 1px solid var(--border);
  flex-shrink: 0;
}

.state-filter-title {
  font-weight: 700;
  font-size: 14px;
  font-family: var(--font-mono);
  color: var(--accent-blue);
  line-height: 20px;
}

.state-filter-list {
  flex: 1;
  overflow-y: auto;
  padding: 8px 8px;
  display: flex;
  flex-direction: column;
  gap: 4px;
}

.state-filter-item {
  display: flex;
  align-items: center;
  justify-content: space-between;
  padding: 4px 12px;
  cursor: pointer;
  font-size: 14px;
  color: #b0b0b8;
  border-radius: 8px;
  transition:
    background 0.15s,
    color 0.15s;
}

.state-filter-item:hover {
  background: rgba(255, 255, 255, 0.05);
  color: var(--text-primary);
}

.state-filter-item.selected {
  background: var(--accent-purple);
  color: #e0d8f0;
}

.state-filter-item.empty {
  color: var(--text-muted);
}

.state-filter-item.empty:hover {
  color: #808088;
}

.state-filter-label {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

.state-filter-count {
  font-family: var(--font-mono);
  font-size: 11px;
  color: var(--text-muted);
  flex-shrink: 0;
  margin-left: 8px;
}

.state-filter-item.selected .state-filter-count {
  color: #9b8cc0;
}
